'use strict';

var vue = require('vue');
var lodashEs = require('lodash-es');
var utils = require('@devops-web/utils');

/**
 * 过滤columns，去掉show===false的列，和children为空的列
 */
function filterColumns(columns) {
    const filteredNodes = [];
    for (const column of columns) {
        if (column.visible !== false) {
            const { children } = column;
            if (Array.isArray(children) && children.length > 0) {
                const filteredChildren = filterColumns(children);
                if (filteredChildren.length > 0) {
                    filteredNodes.push(Object.assign(Object.assign({}, column), { children: filteredChildren }));
                }
            }
            else {
                filteredNodes.push(Object.assign({}, column));
            }
        }
    }
    return filteredNodes;
}
function useColumns(columns) {
    const originColumns = lodashEs.cloneDeep(columns);
    const originColumnsRef = vue.ref(columns);
    const columnsComputed = vue.computed(() => {
        return filterColumns(originColumnsRef.value);
    });
    function resetColumns() {
        originColumnsRef.value = originColumns;
    }
    function replaceColumnByPath(path, column) {
        utils.setValueByPath(originColumnsRef.value, path, column);
    }
    function replaceColumnByDataIndex(dataIndex, column) {
        utils.traverseTreeBFS(originColumnsRef.value, (node) => {
            if (node.dataIndex === dataIndex) {
                Object.keys(node).forEach((key) => {
                    if (Object.prototype.hasOwnProperty.call(column, key)) {
                        // @ts-ignore
                        node[key] = column[key];
                    }
                    else {
                        // @ts-ignore
                        delete node[key];
                    }
                });
            }
        });
    }
    function updateColumnByPath(path, field, value) {
        utils.setValueByPath(originColumnsRef.value, `${path}.${String(field)}`, value);
    }
    function updateColumnByDataIndex(dataIndex, field, value) {
        utils.traverseTreeBFS(originColumnsRef.value, (node) => {
            if (node.dataIndex === dataIndex) {
                node[field] = value;
            }
        });
    }
    function changeVisibleByPath(path, visible) {
        utils.setValueByPath(originColumnsRef.value, `${path}.visible`, visible);
    }
    function changeVisibleByDataIndex(dataIndex, visible) {
        utils.traverseTreeBFS(originColumnsRef.value, (node) => {
            if (node.dataIndex === dataIndex) {
                node.visible = visible;
            }
        });
    }
    return {
        originColumns: originColumnsRef,
        columns: columnsComputed,
        resetColumns,
        changeVisibleByPath,
        changeVisibleByDataIndex,
        updateColumnByPath,
        updateColumnByDataIndex,
        replaceColumnByPath,
        replaceColumnByDataIndex
    };
}

exports.useColumns = useColumns;
